
'use strict';

const {
  getValuesFromProgram,
  getFieldsFromOptions,
  idFromStringOrNum,
  dieWithout
} = require('../lib/utils');

const getModels = require('../lib/getModels')();
const { Note, Task, Sprint, Project } = getModels;

const options = [
  { opt: '-t, --task <string|number>', msg: 'associate note with task' },
  { opt: '-p, --project <string|number>', msg: 'associate note with project' },
  { opt: '-s, --sprint <string|number>', msg: 'associate note with sprint' }
];

async function handler(program) {
  const fields = getFieldsFromOptions(options);
  const values = getValuesFromProgram(fields, program);
  const taskId = await idFromStringOrNum(values.task, Task);
  const projectId = await idFromStringOrNum(values.project, Project);
  const sprintId = await idFromStringOrNum(values.sprint, Sprint);
  const text = program.args[0].join(' ');
  dieWithout({ text }, 'no note text detected. I Quit!');
  const note = await Note.create({ text, taskId, projectId, sprintId });
  return { display: JSON.stringify(note, null, 2) };
}

module.exports = { options, handler };
