
'use strict';

const {
  getValuesFromProgram,
  getFieldsFromOptions,
  dieWithout,
  idFromStringOrNum,
  truthyOrNull
} = require('../lib/utils');

const { Sprint, Task, Project } = require('../lib/getModels')();

const options = [
  { opt: '-i, --instruction <string>', msg: 'your new task' },
  { opt: '-s, --sprint <string|number>', msg: 'the sprint to associate this task with' },
  { opt: '-p, --project <string|number>', msg: 'the project to associate this task with' },
  { opt: '-d, --due <string|number>', msg: 'the due date of the task' },
  { opt: '-e, --estimatedTime <number>', msg: 'an estimate of how long it will take (in minutes)' },
  { opt: '-P, --priority <number>' }
];

async function handler(program) {
  const fields = getFieldsFromOptions(options);
  const values = getValuesFromProgram(fields, program);
  const {
    sprint, project, due,
    estimatedMinsToCompletion,
    instruction, priority
  } = values;

  dieWithout({ instruction });

  const sprintId = await idFromStringOrNum(sprint, Sprint);
  const projectId = await idFromStringOrNum(project, Project);

  const insert = truthyOrNull({
    instruction,
    priority,
    sprintId,
    projectId,
    due,
    estimatedMinsToCompletion
  });

  const task = await Task.create(insert);
  return { display: JSON.stringify(task, null, 2) };
}

module.exports = { options, handler };

