
'use strict';

const {
  getValuesFromProgram,
  getFieldsFromOptions,
  idFromStringOrNum,
  dieWithout
} = require('../lib/utils');

const { Project, Sprint, Task } = require('../lib/getModels')();

const options = [
  {
    opt: '-p, --project <string|number>',
    msg: 'the project to mark as completed'
  },
  {
    opt: '-s, --sprint <string|number>',
    msg: 'the sprint to mark as completed'
  },
  {
    opt: '-t, --task <string|number>',
    msg: 'the task to mark as completed'
  }
];

async function handler(program) {
  const fields = getFieldsFromOptions(options);
  const values = getValuesFromProgram(fields, program);
  const { project, sprint, task } = values;
  let id;
  let Model;
  if (project) {
    id = await idFromStringOrNum(project, Project);
    Model = Project;
  } else if (sprint) {
    id = await idFromStringOrNum(sprint, Sprint);
    Model = Sprint;
  } else if (task) {
    id = await idFromStringOrNum(task, Task);
    Model = Task;
  } else {
    throw new Error('complete requires -t, -s, or -p');
  }

  dieWithout({ id }, 'complete requires a valid -t, -s, or -p option');

  let upd = { where: { id }, values: { completed: Date.now() }};
  const updated = await Model.set(upd);
  return { display: JSON.stringify(updated, null, 2)};
}

module.exports = { options, handler };
