'use strict';

const { getNotes, getNote } = require('../lib/handlers');
const { getValuesFromProgram, getFieldsFromOptions, sortFunc } = require('../lib/utils');
const { oneLineNotes, multiLineNotes } = require('../lib/cli');
const schema = require('../schema/note');

const options = [
  {
    opt: '-n, --note <number>',
    msg: 'find a specific note by it\'s id'
  },
  {
    opt: '-t, --task <string|number>',
    msg: 'find notes from a task'
  },
  {
    opt: '-p, --project <string|number>',
    msg: 'find notes from project'
  },
  {
    opt: '-s, --sprint <string|number>',
    msg: 'find notes from sprint'
  },
  {
    opt: '-H, --header',
    msg: 'displays the column headings (ignored with -m)'
  },
  {
    opt: '-S, --sort <string>',
    msg: 'sorts output by field (prefix with + to reverse)'
  },
  {
    opt: '-c, --count',
    msg: 'returns the number of notes that match'
  },
  {
    opt: '-m, --multi-line',
    msg: 'show the whole note.'
  },
  {
    opt: '-r, --record-separator <string>',
    msg: 'set the string to display between records (only used with -m)'
  }
];

async function handler(program) {
  const fields = getFieldsFromOptions(options);
  const values = getValuesFromProgram(fields, program);
  let { note, count, sort, multiLine, header, recordSeparator } = values;
  let sf = sortFunc(sort, schema);

  let res;
  let ret = {
    display: ''
  };

  if (note && /^[0-9]+$/.test(note)) {
    res = await getNote(note);
    res = [res];
  } else {
    res = await getNotes(values).then(res => res.sort(sf));
  }

  if (count) {
    ret.display = `${res.length}`;
    return ret;
  }

  if(multiLine) {
    let m = multiLineNotes(res, recordSeparator);
    ret.display = m.lines.join('\n');
    return ret;
  }

  let o = oneLineNotes(res, header);
  if (header) {
    o.lines.unshift(o.header);
  }

  ret.display = o.lines.join('\n');

  return ret;
}

module.exports = { options, handler };
