'use strict';

const { getProjects, getProject } = require('../lib/handlers');
const { getValuesFromProgram, getFieldsFromOptions, sortFunc } = require('../lib/utils');
const { oneLineProjects, multiLineProjects } = require('../lib/cli');
const schema = require('../schema/project');

const options = [
  {
    opt: '-p, --project <number>',
    msg: 'find a specific project by it\'s id'
  },
  {
    opt: '-n, --name <string>',
    msg: 'find projects with string in the name'
  },
  {
    opt: '-c, --count',
    msg: 'return the number of projects'
  },
  {
    opt: '-C, --show-completed',
    msg: 'include completed projects ( defaults to false ) (ignored with -n)'
  },
  {
    opt: '-H, --header',
    msg: 'displays the column headings (ignored with -m)'
  },
  {
    opt: '-S, --sort <string>',
    msg: 'sorts output by field (prefix with + to reverse)'
  },
  {
    opt: '-m, --multi-line',
    msg: 'show the whole task.'
  },
  {
    opt: '-N, --notes',
    msg: 'display the notes associated with this project (only used with -m)'
  },
  {
    opt: '-t, --tasks',
    msg: 'display tasks associated with this project (only used with -m)'
  },
  {
    opt: '-r, --record-separator <string>',
    msg: 'set the string to display between records (only used with -m)'
  }
];

async function handler(program) {
  const fields = getFieldsFromOptions(options);
  const values = getValuesFromProgram(fields, program);
  let {
    project,
    multiLine,
    sort,
    count,
    notes,
    recordSeparator,
    showCompleted,
    header,
    tasks
  } = values;

  let sf = sortFunc(sort, schema);
  let res;
  let ret = {
    display: ''
  };

  if (project && /^[0-9]+$/.test(project)) {
    res = await getProject(project);
    res = [res];
  } else {
    res = await getProjects(values).then(res => res.sort(sf));
  }

  if (count) {
    ret.display = `${res.length}`;
    return ret;
  }

  if (multiLine) {
    if (tasks || notes) {
      for (let record of res) {
        await record.populate();
      }
    }
    let m = multiLineProjects(res, notes, tasks, recordSeparator, showCompleted);
    ret.display = m.lines.join('\n');
    return ret;
  }

  for (let record of res) {
    await record.getTaskCount();
  }

  let o = oneLineProjects(res, header);
  if (header) {
    o.lines.unshift(o.header);
  }

  ret.display = o.lines.join('\n');

  return ret;

}


module.exports = { options, handler };
