'use strict';

const { getSprints, getSprint } = require('../lib/handlers');
const {
  getValuesFromProgram,
  getFieldsFromOptions,
  sortFunc
} = require('../lib/utils');
const { oneLineSprints, multiLineSprints } = require('../lib/cli');
const schema = require('../schema/sprint');

const options = [
  {
    opt: '-s, --sprint <number>',
    msg: 'find a specific sprint by it\'s id'
  },
  {
    opt: '-n, --name <string>',
    msg: 'find sprints with string in the name'
  },
  {
    opt: '-c, --count',
    msg: 'return the number of sprints'
  },
  {
    opt: '-C, --show-completed',
    msg: 'include completed sprints ( defaults to false ) (ignored with -n)'
  },
  {
    opt: '-H, --header',
    msg: 'displays the column headings (ignored with -m)'
  },
  {
    opt: '-S, --sort <string>',
    msg: 'sorts output by field (prefix with + to reverse)'
  },
  {
    opt: '-m, --multi-line',
    msg: 'show the whole task.'
  },
  {
    opt: '-N, --notes',
    msg: 'display the notes associated with this sprint (only used with -m)'
  },
  {
    opt: '-t, --tasks',
    msg: 'display tasks associated with this sprint (only used with -m)'
  },
  {
    opt: '-r, --record-separator <string>',
    msg: 'set the string to display between records (only used with -m)'
  }
];

async function handler (program) {
  const fields = getFieldsFromOptions(options);
  const values = getValuesFromProgram(fields, program);
  let {
    sprint,
    multiLine,
    sort,
    count,
    notes,
    recordSeparator,
    header,
    tasks
  } = values;

  let sf = sortFunc(sort, schema);
  // console.log(sf.toString());
  let res;
  let ret = {
    display: ''
  };

  if (sprint && /^[0-9]+$/.test(sprint)) {
    res = await getSprint(sprint);
    res = [res];
  } else {
    res = await getSprints(values).then(res => res.sort(sf));
  }

  if (count) {
    ret.display = `${res.length}`;
    return ret;
  }

  if (multiLine) {
    if (tasks || notes) {
      for (let record of res) {
        await record.populate();
      }
    }
    let m = multiLineSprints(res, notes, tasks, recordSeparator);
    ret.display = m.lines.join('\n');
    return ret;
  }

  for (let record of res) {
    await record.getTaskCount();
  }

  let o = oneLineSprints(res, header);
  if (header) {
    o.lines.unshift(o.header);
  }

  ret.display = o.lines.join('\n');

  return ret;

}

module.exports = { options, handler };
