'use strict';

const { getTasks, getTask } = require('../lib/handlers');
const { oneLineTasks, multiLineTasks } = require('../lib/cli');
const schema = require('../schema/task');

const {
  getValuesFromProgram,
  getFieldsFromOptions,
  sortFunc
} = require('../lib/utils');


const options = [
  {
    opt: '-t, --task <number>',
    msg: 'retrieve a specific task by it\'s id'
  },
  {
    opt: '-s, --sprint <string|number>',
    msg: 'list tasks from this sprint'
  },
  {
    opt: '-p, --project <string|number>',
    msg: 'list tasks from this project'
  },
  {
    opt: '-P, --priority <string|number>',
    msg: 'list tasks with priority (takes +-) and a number'
  },
  {
    opt: '-C, --show-completed',
    msg: 'include completed tasks in output ( defaults to false )'
  },
  {
    opt: '-H, --header',
    msg: 'displays the column headings (ignored with -m)'
  },
  {
    opt: '-S, --sort <string>',
    msg: 'sorts output by field (prefix with + to reverse)'
  },
  {
    opt: '-c, --count',
    msg: 'returns the number of tasks that match'
  },
  {
    opt: '-m, --multi-line',
    msg: 'show the whole task.'
  },
  {
    opt: '-n, --notes',
    msg: 'display the notes associated with this task (only used with -m)'
  },
  {
    opt: '-r, --record-separator <string>',
    msg: 'set the string to display between records (only used with -m)'
  }
];

async function handler(program) {
  const fields = getFieldsFromOptions(options);
  const values = getValuesFromProgram(fields, program);
  const {
    multiLine,
    sort,
    count,
    task,
    notes,
    recordSeparator,
    header
  } = values;

  const sf = sortFunc(sort, schema);
  let res;
  let ret = {
    display: ''
  };

  if (task && /^[0-9]+$/.test(task)) {
    res = await getTask(task);
    res = [res];
  } else {
    res = await getTasks(values).then(res => res.sort(sf));
  }

  if (count) {
    ret.display = `${res.length}`;
    return ret;
  }

  if (multiLine) {
    for (let rec of res) {
      await rec.populate();
    }
    let m = multiLineTasks(res, notes, recordSeparator);
    ret.display = m.lines.join('\n');
    return ret;
  }

  let o = oneLineTasks(res, header);
  if (header) {
    o.lines.unshift(o.header);
  }

  ret.display = o.lines.join('\n');

  return ret;
}

module.exports = exports = { options, handler };
