'use strict';

const { spawn } = require('child_process');

const { getValuesFromProgram, getFieldsFromOptions } = require('../lib/utils');

const options = [
  {
    opt: '-s, --sprint <string|number>',
    msg: 'run tasks from existing sprint'
  },
  {
    opt: '-m, --mode <string>',
    msg: 'display mode: prompt|window|html'
  },
  {
    opt: '-o, --order <string>',
    msg: 'set order of tasks: random|priority|asc|dsc'
  },
  {
    opt: '-g, --go',
    msg: 'don\'t go into interactive mode, just run it (requires -s)'
  }
];

async function handler (program) {
  const fields = getFieldsFromOptions(options);
  const values = getValuesFromProgram(fields, program);
  const { mode, sprint } = values;
  // instead of below, spawn one process that ties the api/electron together
  // give the user back their terminal and be able to kill the api process when
  // the electron process is closed. Also, in this *master* process, generate a
  // key that only lives in it's memory that both the api and electron have to agree
  // on in order to communicate. Thus *guaranteeing* that only the electron process
  // can use the api.
  if (mode === 'window' && sprint) {
    spawn('node', [`${process.cwd()}/commands/run-electron.js`, sprint]);
    process.exit(0);
  }
  return {display:''};
}


module.exports = { options, handler };
