'use strict';

const DB = require('better-sqlite3');
const { sql } = require('../lib/utils');
const { queryString, getOne } = sql();

module.exports = init;

function init(options) {
  return new Driver(options);
}

function Driver(options) {
  const driverOptions = Object.assign({}, options);
  this.name = 'sqlite';
  this.path = driverOptions.path;
  delete driverOptions.path;
  this.db = new DB(this.path, driverOptions);
  this.ignoredValidationPaths = ['id'];
  this.convert = {
    Boolean: String,
    Date: (d) => {
      if (typeof d === 'number') {
        return new Date(d).toISOString().slice(0, 19).replace('T', ' ');
      }
      if (typeof d === 'string' &&
          /^[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}$/.test(d)) {
        return d;
      }
      if (d instanceof Date) {
        return d.toISOString().slice(0, 19).replace('T', ' ');
      }
      throw new Error('Dates Are Hard!');
    }
  };

  removeNamesFromArrowFuncs(this.convert);
}

function removeNamesFromArrowFuncs(obj) {
  for (let key in obj) {
    let val = obj[key].toString();
    if (/=>/.test(val)){
      delete obj[key].name;
    }
  }
}

Driver.prototype.create = async function create(options) {
  let queryOptions = Object.assign({}, options);
  queryOptions.action = 'create';
  const query = queryString(queryOptions);
  const stmt = this.db.prepare(query);
  const { lastInsertRowid } = stmt.run(queryOptions.values);
  return this.get({ type: options.type, where: { id: lastInsertRowid } });
};

Driver.prototype.get = async function get(options) {
  let queryOptions = Object.assign({}, options);
  queryOptions.action = 'get';
  const query = queryString(queryOptions);
  const stmt = this.db.prepare(query);
  if (getOne(queryOptions)) {
    return stmt.get();
  } else {
    return stmt.all();
  }
};

Driver.prototype.set = async function set(options) {
  let queryOptions = Object.assign({}, options);
  let { where } = queryOptions;
  queryOptions.action = 'set';
  const query = queryString(queryOptions);
  const stmt = this.db.prepare(query);
  stmt.run();
  return this.get({ type: queryOptions.type, where });
};

Driver.prototype.remove = async function remove(options) {
  let queryOptions = Object.assign({}, options);
  queryOptions.action = 'remove';
  const query = queryString(queryOptions);
  const stmt = this.db.prepare(query);
  return stmt.run();
};

Driver.prototype.close = function close() {
  this.db.close();
};
