'use strict';

const loadDriver = require('./loadDriver');
const Model = require('./model');
const Schema = require('./schema');
const { throwWithout } = require('./utils');
const taskSchema = require('../schema/task');
const projectSchema = require('../schema/project');
const sprintSchema = require('../schema/sprint');
const noteSchema = require('../schema/note');
const taskMixins = require('../mixins/task');
const tInstMix = require('../mixins/task-instance');
const projectMixins = require('../mixins/project');
const pInstMix = require('../mixins/project-instance');
const sprintMixins = require('../mixins/sprint');
const sInstMix = require('../mixins/sprint-instance');
const noteMixins = require('../mixins/note');
const nInstMix = require('../mixins/note-instance');

module.exports = Gishdo;

function Gishdo(options) {
  this.models = {};
  this.driver = loadDriver(options);
  this.model = model;
  this.getModelNames = getModelNames;
  this.hasModel = hasModel;
  this.model('task', taskSchema, taskMixins, tInstMix(this));
  this.model('project', projectSchema, projectMixins, pInstMix(this));
  this.model('sprint', sprintSchema, sprintMixins, sInstMix(this));
  this.model('note', noteSchema, noteMixins, nInstMix(this));
}

function model(name, schema, mixins, instanceMixins) {
  throwWithout({ name });

  if (!this.hasModel(name)) {
    throwWithout({ schema });
    let { ignoredValidationPaths, convert } = this.driver;
    let s = new Schema(schema, ignoredValidationPaths, convert);
    let model = new Model(name, s, this.driver, instanceMixins);
    for (let method of Object.keys(mixins)) {
      model[method] = mixins[method];
    }
    this.models[name] = model;
  }

  return this.models[name];
}

function getModelNames() {
  return Object.keys(this.models);
}

function hasModel(n) {
  return Object.keys(this.models).includes(n);
}
