'use strict';

module.exports = Model;

function Model(name, schema, driver, instanceMixins) {
  this.type = name;
  this.schema = schema;
  this.create = create.bind(driver, this.type, this.schema, imbue);
  this.get = get.bind(driver, this.type, this.schema, imbue);
  this.set = set.bind(driver, this.type, this.schema, imbue);
  this.remove = remove.bind(driver, this.type, this.schema);

  function imbue(obj) {
    if (!Array.isArray(obj)) {
      for (let method in instanceMixins) {
        obj[method] = instanceMixins[method];
      }
    } else {
      obj.map(el => {
        for (let method in instanceMixins) {
          el[method] = instanceMixins[method];
        }
      });
    }
    return obj;
  }
}

function create( type, schema, mixins, object) {
  let obj = { type };
  const { castIn, castOut, setDefaults, validate } = schema;
  obj.values = castIn(setDefaults(Object.assign({}, object)));
  let e = validate(obj.values);
  if (e instanceof Error) {
    return Promise.reject(e);
  }
  return this.create(obj).then(castOut).then(mixins);
}

function get(type, { castIn, castOut }, mixins, object) {
  let obj = { type };
  Object.assign(obj, object);
  // this was obtuse. of course you should be able to query without where.
  // if (!obj.where) {
  //   return Promise.reject(new Error('Model.get requires a where property'));
  // }
  obj.where = castIn(obj.where);
  return this.get(obj).then(castOut).then(mixins);
}

function set(type, { castIn, castOut }, mixins, object) {
  let obj = { type };
  Object.assign(obj, object);
  if (!obj.where) {
    return Promise.reject(new Error('Model.set requires a where property'));
  }
  obj.where = castIn(obj.where);
  obj.values = castIn(obj.values);
  return this.set(obj).then(castOut).then(mixins);
}

function remove(type, { castIn }, object) {
  let obj = { type };
  Object.assign(obj, object);
  if (!obj.where) {
    return Promise.reject(new Error('Model.remove requires a where property'));
  }
  obj.where = castIn(obj.where);
  return this.remove(obj);
}
